##############################################################################
# Copyright (c) 1999, Tres Seaver, Palladion Software
# ---------------------------------------------------
#
# Please see the enclosed LICENSE.txt for a statement of the terms under
# which this software is available to you.
##############################################################################

import rfc822
import string
import StringIO

#
#   Load DTML methods.
#
addForm = view = properties = normal = all = brief = select = body = None
try:    # work around testing outside of Zope
    from Globals import HTMLFile

    addForm     = HTMLFile( 'addMessage',       globals() )
    view        = HTMLFile( 'viewMessage',      globals() )
    properties  = HTMLFile( 'manageMessage',    globals() )
    normal      = HTMLFile( 'normalHeaders',    globals() )
    brief       = HTMLFile( 'briefHeaders',     globals() )
    all         = HTMLFile( 'allHeaders',       globals() )
    select      = HTMLFile( 'selectHeaders',    globals() )
    body        = HTMLFile( 'messageBody',      globals() )
except:
    print 'Couldn''t import HTMLFile!!'

#
#   Set up Zope base class
#
try:    # work around testing outside of Zope
    from Globals import Persistent
    import OFS.SimpleItem, Acquisition, AccessControl.Role

    class ZopeBase( OFS.SimpleItem.Item
                  , Persistent
                  , Acquisition.Implicit
                  , AccessControl.Role.RoleManager
                  ):
        pass
except:
    class ZopeBase: pass
    print 'Couldn''t import Zope base classes!!'

#
#   Module-level constructors
#
manage_addMailMessageForm = addForm
def manage_addMailMessage( self, id, file='', REQUEST=None ):
    """
    """
    if not file:
        if REQUEST:
            file = REQUEST[ 'messageFile' ]
        else:
            raise BadRequest, 'No file!' 

    if type( file ) == type( '' ):
        fp = StringIO.StringIO( file )
    else:
        fp = file
    msg = MailMessage( id, fp, 1 )
    self._setObject( id, msg )
    if REQUEST is not None:
       return self.manage_main( self, REQUEST )
     

class MailMessage( ZopeBase):
    """
        Capture an RFC822-compatible mail message as a Zope object.
    """
    meta_type = 'MailMessage'
    manage_options = ( { 'label' : 'Properties',    'action' : 'manage_main' }
                     , { 'label' : 'View',          'action' : 'index_html' }
                     , { 'label' : 'Security',      'action' : 'manage_access' }
                     )

    __ac_permissions__=( ( 'View management screens', ( 'manage_main',    ) )
                       , ( 'View',                    ( '__call__',    '' ) )
                       ,
                       )

    manage_main     = properties
    index_html      = view
    normalHeaders   = normal
    briefHeaders    = brief
    allHeaders      = all
    selectHeaders   = select
    messageBody     = body

    def __init__( self, id, fp, seekable=0 ):
        """
            Parse a stream-like object, fp, as an RFC822-compatible message,
            including the body lines.
        """
        self.id             = id
        self.name           = id
        hdrs = rfc822.Message( fp, seekable )
        self.headers        = hdrs.headers[:]
        self.headerList     = self._normalizeHeaders( hdrs.headers )
        self.unixfrom       = hdrs.unixfrom
        self.messageDate    = hdrs.getdate_tz( 'Date' )
        self.fromAddress    = hdrs.getaddr( 'From' )
        self.toAddresses    = hdrs.getaddrlist( 'To' )
        self.ccAddresses    = hdrs.getaddrlist( 'Cc' )
        self.subject        = hdrs.dict.get( 'subject', '<no subject>' )
        self.bodyLines      = []
        for line in fp.readlines():
            self.bodyLines.append( line )
    
    def title( self ):
        """
        """
        return self.subject

    def document_src( self ):
        """
            Reconstruct the message.
        """
        lines = string.split( self.unixfrom, '\n' ) + ['\n'] \
              + self.headers
        return string.join( lines, '' ) + '\r\n' \
              + string.join( self.bodyLines, '' )

    def _normalizeHeaders( self, rfcHeaders ):
        result       = []
        currentHdr   = ''
        currentLines = []
        for line in rfcHeaders:
            if line[0] in '\t ':
                currentLines.append( line )
            else:
                if currentLines:
                    result.append( ( currentHdr, currentLines ) )
                tokens = string.split( line, ':' )
                currentHdr      = tokens[ 0 ]
                currentLines = [ string.join( tokens[ 1: ], ':' ) ]
        if currentLines:
            result.append( ( currentHdr, currentLines ) )
        return result


if __name__ == '__main__':
    import sys, string
    sys.path.insert( 0, r'E:\tools\NTZope\lib\python' )
    msg = MailMessage( 0, sys.stdin )
    print msg.document_src()
    fmtstr = 'messageDate: %(messageDate)s\n' \
           + 'subject:     %(subject)s\n' \
           + 'toAddresses: %(toAddresses)s\n' \
           + 'ccAddresses: %(ccAddresses)s\n'
    sys.stderr.write( fmtstr % msg.__dict__ )
    for hdr in msg.headerList:
        sys.stderr.write( '[' + str( hdr ) + ']\n' )
